//
//  View.m
//  Heading
//
//  Created by NYU User on 10/12/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "View.h"


@implementation View


- (id) initWithFrame: (CGRect) frame {
	if ((self = [super initWithFrame: frame])  != nil) {
		// Initialization code
		self.backgroundColor = [UIColor whiteColor];
		
		manager = [[CLLocationManager alloc] init];
		manager.delegate = self;
		manager.headingFilter = kCLHeadingFilterNone; //report every change
		manager.headingOrientation = CLDeviceOrientationPortrait;
		[manager startUpdatingHeading];
		
	}
	return self;
}


// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void) drawRect: (CGRect) rect {
	// Drawing code
	UIFont *f = [UIFont systemFontOfSize: 20];
	CGPoint p = CGPointMake(5, 0);
	
	NSString *s = [NSString stringWithFormat: @"magnetic == %f°", 
		magneticHeading];
	[s drawAtPoint: p withFont: f];
	CGFloat height = [s sizeWithFont: f].height;
	p.y += height;	//Move down one line.
	
	s = [NSString stringWithFormat: @"true == %f°",
		 trueHeading];
	[s drawAtPoint: p withFont: f];
	p.y += height;
	
	s = [NSString stringWithFormat: @"accuracy == %f°",
		 headingAccuracy];
	[s drawAtPoint: p withFont: f];
	p.y += height;
	
	s = [NSString stringWithFormat: @"x == %f µT", x];
	[s drawAtPoint: p withFont: f];
	p.y += height;
	
	s = [NSString stringWithFormat: @"y == %f µT", y];
	[s drawAtPoint: p withFont: f];
	p.y += height;

	s = [NSString stringWithFormat: @"z == %f µT", z];
	[s drawAtPoint: p withFont: f];
	
	//Put the origin at the center.
	CGContextRef c = UIGraphicsGetCurrentContext();
	CGSize size = self.bounds.size;
	CGContextTranslateCTM(c, size.width / 2, size.height / 2);
	
	//X axis
	CGContextBeginPath(c);
	CGContextMoveToPoint(c, -size.width / 2, 0);
	CGContextAddLineToPoint(c, size.width / 2, 0);
	
	//Y axis
	CGContextMoveToPoint(c, 0, size.height / 2);
	CGContextAddLineToPoint(c, 0, -size.height / 2);

	CGFloat radius = 1;
	CGFloat width = 2 * radius;
	CGFloat scale = size.width / width;
	CGContextConcatCTM(c, CGAffineTransformMakeScale(scale, -scale));
	
	CGRect r = CGRectMake(-radius, -radius, 2 * radius, 2 * radius);
	CGContextAddEllipseInRect(c, r);
	
	CGContextSetRGBStrokeColor(c, 0, 0, 1, 0.5);	//faint blue
	CGContextSetLineWidth(c, 1.0 / scale);
	CGContextStrokePath(c);

	//compass needle pointing towards magnetic north
	CLLocationDirection theta = (90 - magneticHeading) * M_PI / 180;
	CGContextBeginPath(c);
	CGContextMoveToPoint(c, 0, 0);
	CGContextAddLineToPoint(c, cos(theta), sin(theta));
	CGContextSetRGBStrokeColor(c, 1, 0, 0, 1);	//needle traditionally red
	CGContextStrokePath(c);
}


- (void) dealloc {
	[manager release];
	[super dealloc];
}

#pragma mark -
#pragma marK protocol CLLocationManagerDelegate

- (void) locationManager: (CLLocationManager *) m
	didUpdateHeading: (CLHeading *) newHeading {
	
	if (m == manager) {
		magneticHeading = newHeading.magneticHeading;
		trueHeading = newHeading.trueHeading;
		headingAccuracy = newHeading.headingAccuracy;

		x = newHeading.x;
		y = newHeading.y;
		z = newHeading.z;
		
		[self setNeedsDisplay];
	}
}

- (void) locationManager: (CLLocationManager *) m
	didFailWithError: (NSError *) error {
	NSLog(@"Location manager failed with error: %@", error);
}
@end
